# -*- coding: iso-8859-1 -*-
# Copyright (C) 2004-2010,2014 Bastian Kleineidam
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""
Test news checking.
"""
import pytest
from tests import need_newsserver, limit_time
from . import LinkCheckTest

# Changes often, as servers tend to get invalid. Thus it is necessary
# to enable the has_newsserver() resource manually.
NNTP_SERVER = "news.uni-stuttgart.de"
# info string returned by news server
NNTP_INFO = (
    "200 news.uni-stuttgart.de InterNetNews NNRP server INN 2.5.2 ready (no posting)"
)
# Most free NNTP servers are slow, so don't waist a lot of time running those.
NNTP_TIMEOUT_SECS = 30


# disabled for now until some stable news server comes up
@pytest.mark.skip(reason="disabled for now until some stable news server comes up")
class TestNews(LinkCheckTest):
    """Test nntp: and news: link checking."""

    def newstest(self, url, resultlines):
        self.direct(url, resultlines)

    def test_news_without_host(self):
        # news testing
        url = "news:comp.os.linux.misc"
        resultlines = [
            "url %s" % url,
            "cache key %s" % url,
            "real url %s" % url,
            "warning No NNTP server was specified, skipping this URL.",
            "valid",
        ]
        self.newstest(url, resultlines)
        # no group
        url = "news:"
        resultlines = [
            "url %s" % url,
            "cache key %s" % url,
            "real url %s" % url,
            "warning No NNTP server was specified, skipping this URL.",
            "valid",
        ]
        self.newstest(url, resultlines)

    def test_snews_with_group(self):
        url = "snews:de.comp.os.unix.linux.misc"
        nurl = self.norm(url)
        resultlines = [
            "url %s" % url,
            "cache key %s" % nurl,
            "real url %s" % nurl,
            "warning No NNTP server was specified, skipping this URL.",
            "valid",
        ]
        self.newstest(url, resultlines)

    def test_illegal_syntax(self):
        # illegal syntax
        url = "news:§$%&/´`(§%"
        qurl = self.norm(url)
        resultlines = [
            "url %s" % url,
            "cache key %s" % qurl,
            "real url %s" % qurl,
            "warning No NNTP server was specified, skipping this URL.",
            "valid",
        ]
        self.newstest(url, resultlines)

    @need_newsserver(NNTP_SERVER)
    @limit_time(NNTP_TIMEOUT_SECS, skip=True)
    def test_nntp_with_host(self):
        url = "nntp://%s/comp.lang.python" % NNTP_SERVER
        resultlines = [
            "url %s" % url,
            "cache key %s" % url,
            "real url %s" % url,
            "info %s" % NNTP_INFO,
            "info News group comp.lang.python found.",
            "valid",
        ]
        self.newstest(url, resultlines)

    @need_newsserver(NNTP_SERVER)
    @limit_time(NNTP_TIMEOUT_SECS, skip=True)
    def test_article_span(self):
        url = "nntp://%s/comp.lang.python/1-5" % NNTP_SERVER
        resultlines = [
            "url %s" % url,
            "cache key %s" % url,
            "real url %s" % url,
            "info %s" % NNTP_INFO,
            "info News group comp.lang.python found.",
            "valid",
        ]
        self.newstest(url, resultlines)

    def test_article_span_no_host(self):
        url = "news:comp.lang.python/1-5"
        resultlines = [
            "url %s" % url,
            "cache key %s" % url,
            "real url %s" % url,
            "warning No NNTP server was specified, skipping this URL.",
            "valid",
        ]
        self.newstest(url, resultlines)

    @need_newsserver(NNTP_SERVER)
    @limit_time(NNTP_TIMEOUT_SECS, skip=True)
    def test_host_no_group(self):
        url = "nntp://%s/" % NNTP_SERVER
        resultlines = [
            "url %s" % url,
            "cache key %s" % url,
            "real url %s" % url,
            "info %s" % NNTP_INFO,
            "warning No newsgroup specified in NNTP URL.",
            "valid",
        ]
        self.newstest(url, resultlines)



# Copyright (C) 2004-2014 Bastian Kleineidam
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""
Test https.
"""
from unittest.mock import patch

from OpenSSL import crypto

from .httpserver import HttpsServerTest, CookieRedirectHttpRequestHandler
from .. import get_file

from linkcheck import httputil


class TestHttps(HttpsServerTest):
    """
    Test https: link checking.
    """

    def __init__(self, methodName="runTest"):
        super().__init__(methodName=methodName)
        self.handler = CookieRedirectHttpRequestHandler

    @classmethod
    def setUpClass(cls):
        key = crypto.PKey()
        key.generate_key(crypto.TYPE_RSA, 2048)
        cert = crypto.X509()
        cert.get_subject().CN = "localhost"
        cert.add_extensions(
            [crypto.X509Extension(b"subjectAltName", False, b"DNS:localhost")])
        cert.set_serial_number(1000)
        cert.gmtime_adj_notBefore(0)
        cert.set_notAfter(b"21190102030405Z")
        cert.set_issuer(cert.get_subject())
        cert.set_pubkey(key)
        cert.sign(key, "sha1")
        with open(get_file("https_key.pem"), "wb") as f:
            f.write(crypto.dump_privatekey(crypto.FILETYPE_PEM, key))
        with open(get_file("https_cert.pem"), "wb") as f:
            f.write(crypto.dump_certificate(crypto.FILETYPE_PEM, cert))

    def test_https(self):
        url = self.get_url("")
        resultlines = [
            "url %s" % url,
            "cache key %s" % url,
            "real url %s" % url,
            "valid",
        ]
        confargs = dict(sslverify=False)
        with patch.dict("os.environ",
                        {"REQUESTS_CA_BUNDLE": get_file("https_cert.pem")}):
            self.direct(url, resultlines, recursionlevel=0, confargs=confargs)

    def test_x509_to_dict(self):
        with open(get_file("https_cert.pem"), "rb") as f:
            cert = crypto.load_certificate(crypto.FILETYPE_PEM, f.read())
        self.assertEqual(
            httputil.x509_to_dict(cert)["notAfter"], "Jan 02 03:04:05 2119 GMT"
        )



# Copyright (C) 2004-2012 Bastian Kleineidam
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""
Test telnet checking.
"""
from .telnetserver import TelnetServerTest


class TestTelnet(TelnetServerTest):
    """Test telnet: link checking."""

    def test_telnet_error(self):
        url = "telnet:"
        nurl = self.norm(url)
        resultlines = [
            "url %s" % url,
            "cache key None",
            "real url %s" % nurl,
            "error",
        ]
        self.direct(url, resultlines)

    def test_telnet_localhost(self):
        url = self.get_url()
        resultlines = [
            "url %s" % url,
            "cache key %s" % url,
            "real url %s" % url,
            "valid",
        ]
        self.direct(url, resultlines)
        url = self.get_url(user="test")
        resultlines = [
            "url %s" % url,
            "cache key %s" % url,
            "real url %s" % url,
            "valid",
        ]
        self.direct(url, resultlines)
        url = self.get_url(user="test", password="test")
        resultlines = [
            "url %s" % url,
            "cache key %s" % url,
            "real url %s" % url,
            "valid",
        ]
        self.direct(url, resultlines)



# Copyright (C) 2012 Bastian Kleineidam
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""
Define http test support classes for LinkChecker tests.
"""
import time
import threading
import telnetlib
import miniboa
from . import LinkCheckTest


TIMEOUT = 5


class TelnetServerTest(LinkCheckTest):
    """Start/stop a Telnet server that can be used for testing."""

    def __init__(self, methodName="runTest"):
        """Init test class and store default ftp server port."""
        super().__init__(methodName=methodName)
        self.host = "localhost"
        self.port = None
        self.stop_event = threading.Event()
        self.server_thread = None

    def get_url(self, user=None, password=None):
        if user is not None:
            if password is not None:
                netloc = f"{user}:{password}@{self.host}"
            else:
                netloc = f"{user}@{self.host}"
        else:
            netloc = self.host
        return "telnet://%s:%d" % (netloc, self.port)

    def setUp(self):
        """Start a new Telnet server in a new thread."""
        super().setUp()
        self.port, self.server_thread = start_server(self.host, 0, self.stop_event)
        self.assertFalse(self.port is None)

    def tearDown(self):
        """Send QUIT request to telnet server."""
        self.stop_event.set()
        if self.server_thread is not None:
            self.server_thread.join(10)
            assert not self.server_thread.is_alive()


def start_server(host, port, stop_event):
    # Instantiate Telnet server class and listen to host:port
    clients = []

    def on_connect(client):
        clients.append(client)
        client.send("Telnet test server\nlogin: ")

    server = miniboa.TelnetServer(port=port, address=host, on_connect=on_connect)
    port = server.server_socket.getsockname()[1]
    t = threading.Thread(None, serve_forever, args=(server, clients, stop_event))
    t.start()
    # wait for server to start up
    tries = 0
    while tries < 5:
        tries += 1
        try:
            client = telnetlib.Telnet(timeout=TIMEOUT)
            client.open(host, port)
            client.write(b"exit\n")
            break
        except Exception:
            time.sleep(0.5)
    return port, t


def serve_forever(server, clients, stop_event):
    """Run poll loop for server."""
    while True:
        if stop_event.is_set():
            return
        server.poll()
        for client in clients:
            if client.active and client.cmd_ready:
                handle_cmd(client)


def handle_cmd(client):
    """Handle telnet clients."""
    msg = client.get_command().lower()
    if msg == "exit":
        client.active = False
    else:
        client.send("Password: ")
